/** @param {!Array<_argufy.Flag>} flags */
const serialiseFlags = (flags) => {
  let obj = flags.map(({
    name, short, boolean, number, command, multiple, description, default: def,
  }) => {
    // let codes = {}
    const d = description
      // .replace(/`(.+)`/g, (m, code, i) => {
      //   codes[i] = code
      //   return `%%RESTREAM_REPLACE_${i}%%`
      // })
      .replace(/'/g, '\\\'').replace(/\n\s*/g, '\\n')
      // .replace(/%%RESTREAM_REPLACE_(\d+)%%/g, (m, i) => {
      //   return '`' + codes[i] + '`'
      // })
    let s = []
    if (description) s.push(`description: '${d}'`)
    if (command) s.push('command: true')
    if (multiple) s.push('multiple: true')
    if (boolean) s.push('boolean: true')
    if (number) s.push('number: true')
    if (def) s.push(`default: '${def}'`)
    if (!s.length && short) {
      return `'${name}': '${short}'`
    } else if (!s.length && !short) {
      return `'${name}': { }`
    } else if (short) {
      s.push(`short: '${short}'`)
    }
    return `'${name}': {
  ${s.join(',\n  ')}${s.length ? ',' : ''}
}`
  }).join(',\n')
  if (obj) obj = `${obj},`
  obj = obj.replace(/^/mg, '  ')
  return `{
${obj}
}`
}

/**
 * @param {!Array<_argufy.Flag>} args
 * @param {string} argsName
 */
const makeExports = (args, lastVarArgsName, argsName = '') => {
  const obj = serialiseFlags(args)
  const varConfigName = `argsConfig${argsName}`
  const varArgsName = `args${argsName}`
  const argExports = args.map(arg => makeExport(arg, varArgsName))
  return { varArgsName, string: `
export const ${varConfigName} = ${obj}
const ${varArgsName} = argufy(${varConfigName}${
  lastVarArgsName ? `, [process.argv[0], process.argv[1], ...${lastVarArgsName}._argv]` : ''
})

${argExports.join('\n\n')}
` }
}

/** @param {_argufy.Flag} f */
const makeExport = ({ name, number, boolean, description, default: def, multiple }, argsVar = 'args') => {
  let type = 'string'
  if (number) type = 'number'
  if (boolean) type = 'boolean'
  if (multiple) type = '(!Array<string>|string)'
  const n = name.replace(/-(.)/g, (m, l) => l.toUpperCase())
  let ddef = `'${def}'`
  if (number || boolean) ddef = def
  const hasDef = def !== undefined
  // const ddef = number ? def :
  return `/**
 * ${description}${hasDef ? ` Default \`${def}\`.` : ''}
 */
export const _${n} = /** @type {${type}} */ (${argsVar}['${name}']${
  hasDef ? ` || ${ddef}` : ''})`
}

/**
 * @suppress {nonStandardJsDocs}
 * @typedef {import('.').Flag} _argufy.Flag
 */

module.exports.serialiseFlags = serialiseFlags
module.exports.makeExports = makeExports