import { readFileSync, writeFileSync } from 'fs'
import rexml from 'rexml'
import usage from './usage'
import { reduceUsage } from './'
import { _input, _output, _help, _version, argsConfig } from './get-args'
import { makeExports } from './lib'

if (_help) {
  const a = reduceUsage(argsConfig)
  console.log(usage(a))
  process.exit()
} else if (_version) {
  console.log(require('../package.json')['version'])
  process.exit()
}

/**
 * Read the arguments.xml file and return flags.
 * @param {string} input
 */
const extract = (input) => {
  const xml = `${readFileSync(input)}`
  const root = rexml('arguments', xml)
  if (!root.length)
    throw new Error('XML file should contain root arguments element.')

  const [{ content, props: { 'name': argumentsName } }] = root

  const allArgs = rexml('arg', content)
  /** @type {!Array<!_argufy.Flag>} */
  const args = allArgs.map(({ content: description, props: {
    'command': command,
    'name': name,
    'boolean': boolean,
    'number': number,
    'multiple': multiple,
    'short': short,
    'default': def,
  } }) => {
    return {
      description: description.trim(),
      command, boolean, number, multiple, short, name, default: def,
    }
  })
  return { argumentsName, args }
}

/**
 * @param {string} allExports
 * @param {string} lastVarArgsName
 */
const template = (allExports, lastVarArgsName) => {
  return `import argufy from 'argufy'
${allExports}
/**
 * The additional arguments passed to the program.
 */
export const _argv = /** @type {!Array<string>} */ (${lastVarArgsName}._argv)`
}

try {
  const inputs = Array.isArray(_input) ? _input : [_input]
  let lastVarArgsName = ''
  const res = inputs.map((input) => {
    const e = extract(input)
    return e
  }).reduce((acc, { argumentsName = '', args }) => {
    const { string, varArgsName } = makeExports(args, lastVarArgsName, argumentsName)
    lastVarArgsName = varArgsName
    acc.push(string)
    return acc
  }, [])
  const allExports = res.join('')
  debugger
  const t = template(allExports, lastVarArgsName)
  if (_output) {
    writeFileSync(_output, t)
    console.log('Output saved to %s', _output)
  } else console.log(t)
} catch (err) {
  process.env['DEBUG'] ? console.log(err.stack) : console.log(err.message)
}

/**
 * @suppress {nonStandardJsDocs}
 * @typedef {import('.').Flag} _argufy.Flag
 */